<?php
namespace Xoogu\CurrencyConversion;
class Ajax_Responder {
	
    //store the mysql table names
    protected $_tblCur, $_tblCou;
    //holds CountryFromIP object used for Geo IP
	protected $_geoIP;
	//Database connection
	protected $_wpdb;
	
	/**
	 *Loads the main plugin settings from the db so the correct geoIP method can be used and initialises an instance of CountryFromIP for use in looking up the user's country from their IP address
	 */
	public function __construct(\wpdb $wpdb, \Xoogu\CountryFromIP\ICountryFromIP $countryFromIP)
	{
		//get the main plugin settings so we can use the correct geoIP method
		//load our geoip class
		$this->_geoIP = $countryFromIP;
		$this->_wpdb = $wpdb;
        //mysql table names
        $this->_tblCur = $wpdb->base_prefix.'xoogu_currency_details';
        $this->_tblCou = $wpdb->base_prefix.'xoogu_currency_country';
	}
	
	/**
	 *Gets currency and country information based on request information sent as a JSON object on the request body
	 *Outputs the requested information as JSON then terminates the script
	 */
	public function ajaxResponse()
	{
		//get the request body
		$oFetchInfoFor = json_decode(file_get_contents('php://input'));
		//return the info as JSON
		header('Content-type: application/json');
		echo json_encode($this->getInfo($oFetchInfoFor));
		exit();
	}
	
	/**
	 *Gets currency and country information based on the values in the provided object
	 *@param object $oFetchInfoFor What currencies and countries the information should be retieved for.
	 *@return object The currency and country information
	 */
	public function getInfo(\stdClass $oFetchInfoFor)
	{
		//declare vars
		$userCountry=null; $userCurrency=null;
		//if a user currency was specified, add it to the list of currencies to get info for
		if (!empty($oFetchInfoFor->currencyCode) && strlen($oFetchInfoFor->currencyCode) === 3) {
			$userCurrency = $oFetchInfoFor->currencyCode;
			if (empty($oFetchInfoFor->currencies) || !is_array($oFetchInfoFor->currencies)) {
				$oFetchInfoFor->currencies = array($userCurrency);
			} elseif (!in_array($userCurrency, $oFetchInfoFor->currencies)) {
				$oFetchInfoFor->currencies[] = $userCurrency;
			}
		}
		//look up the user's country from their IP address if a user country was not specified
		if (empty($oFetchInfoFor->countryCode) || strlen($oFetchInfoFor->countryCode) !== 2) {
			if (!$userCurrency) {
				$userCountry = $this->_geoIP->getCountryFromIP();
			}
		} else {
			$userCountry = $oFetchInfoFor->countryCode;
		}
		if ($userCountry) {
			//add their country to the list of countries to retrieve currency info for
			if (empty($oFetchInfoFor->countries) || !is_array($oFetchInfoFor->countries)) {
				$oFetchInfoFor->countries = array($userCountry);
			} elseif (!in_array($userCountry, $oFetchInfoFor->countries)) {
				$oFetchInfoFor->countries[] = $userCountry;
			}
		}
		//get the country specific currency info (for formatting)
		$aCountryInfo = $this->getCountryInfo($oFetchInfoFor);
		//get the currency info
		$aCurrencyInfo = $this->getCurrencyInfo($oFetchInfoFor);
		//create a map for looking up currencies by their symbol
		$aCurrenciesBySymbol = array();
		if (!empty($aCurrencyInfo)) {
			foreach ($aCurrencyInfo as $oCurrencyInfo) {
				if ($oCurrencyInfo->symbol_pref) {
					$aCurrenciesBySymbol[$oCurrencyInfo->symbol] =  $oCurrencyInfo->code;
				}
			}
		}
		//get static rates
		$aStaticRates = array();
		if (!empty($oFetchInfoFor->staticRates)) {
			//fetch static rates from db
			//for each currency add a staticRate prop with the value
			$aStaticRates = $this->getStaticRates($oFetchInfoFor->staticRates, $aCurrencyInfo);
		}
		//Associative arrays will always be cast to an object in JSON
		//So we'll cast them to an object here too, to provide consistency
		$returnVal =  (object)array(
			'currencyCode' => $userCurrency ? $userCurrency : ($userCountry ? $aCountryInfo[$userCountry]->currency : null),
			'countryCode' => $userCountry ? $userCountry : ($userCurrency ? $aCurrencyInfo[$userCurrency]->format_pref : null),
			'countries' => (object)$aCountryInfo,
			'currencies' => (object)$aCurrencyInfo,
			'currenciesBySymbol' => (object)$aCurrenciesBySymbol,
			'staticRates' => (object)$aStaticRates
		);
		return $returnVal;
	}
	
	/**
	 *Adds static rate information to the provided currency detail objects, based on the provided post ID
	 *@param array $aPostIDs The posts to retrieve static rates data for (the rates data must have previously been saved as a meta option for the posts)
	 *@param array $aCurrencyInfo Array of currency info objects
	 *@return array Array of currency info objects with staticRate property added
	 */
	public function getStaticRates(array $aPostIDs, array $aCurrencyInfo)
	{
		$aStaticRates = array();
		if (empty($aPostIDs)) {
			return $aStaticRates;
		}
		foreach ($aPostIDs as $postID) {
			$aStaticRates[$postID] = array();
			$aRatesForPost = get_metadata('post', $postID, '_xoogu_currency_conversion_rates', true);
			//get_post_meta($postID, '_xoogu_currency_conversion_rates', true);
			foreach ($aCurrencyInfo as $oCurrencyInfo) {
				$aStaticRates[$postID][$oCurrencyInfo->code] = floatval($aRatesForPost[$oCurrencyInfo->code]);
			}
		}
		return $aStaticRates;
	}
	
	public function getCurrencyInfo(\stdClass $oFetchInfoFor)
	{
		//build the SQL query to get the currency details we want
		$sSQL = $sSQLStart = 'SELECT DISTINCT code, symbol, name, full_name, dec_places, symbol_pref, format_pref, rate
		FROM '.$this->_tblCur.' WHERE';
		
		//We build the query with %s placeholders for the values, then insert the values using wpdb::prepare
		//So we need an array to hold all our values to replace the placeholders with
		$aValsToInsertInQuery = array();
			
		if (!empty($oFetchInfoFor->currenciesBySymbol)) {
			$sSQL .= ' (symbol IN ('.str_repeat('%s,',count($oFetchInfoFor->currenciesBySymbol)-1).'%s) AND symbol_pref = 1)';
			$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->currenciesBySymbol);
		}
		if (!empty($oFetchInfoFor->currencies)) {
			if (!empty($oFetchInfoFor->currenciesBySymbol)) {
				$sSQL .= ' OR';
			}
			$sSQL .= ' code IN ('.str_repeat('%s,',count($oFetchInfoFor->currencies)-1).'%s)';
			$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->currencies);
		}
		if (!empty($oFetchInfoFor->countries)) {
			if (!empty($oFetchInfoFor->currencies) || !empty($oFetchInfoFor->currenciesBySymbol)) {
				$sSQL .= ' UNION '.$sSQLStart;
			}
			$sSQL .= ' code IN ( SELECT currency FROM '.$this->_tblCou.'  WHERE country IN ('.str_repeat('%s,',count($oFetchInfoFor->countries)-1).'%s) )';
			$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->countries);
		}
		
		if (empty($aValsToInsertInQuery)) {
			return;
		}
		$sSQL = $this->_wpdb->prepare($sSQL, $aValsToInsertInQuery);
		$aResults = $this->_wpdb->get_results($sSQL, OBJECT_K);
		foreach ($aResults as &$oResult) {
			$oResult->dec_places = intval($oResult->dec_places);
			$oResult->symbol_pref = (bool) $oResult->symbol_pref;
			$oResult->rate = floatval($oResult->rate);
		}
		return $aResults;
	}
	
	/**
	 *Gets the country specific currency info (for formatting)
	 *@param object $oFetchInfoFor Object containing one or more of the following public properties: currenciesBySymbol, currencies, countries. Each property should be an array containing one or more of the relevant items for which the information is wanted
	 *@return array The country specific currency information for the requested currencies / countries
	 */
	public function getCountryInfo(\stdClass $oFetchInfoFor)
	{
		//build the SQL query to get the country details we want
		$sSQL = 'SELECT c.country, c.currency, c.dec_point, c.thousands_sep, c.symbol_pos
		FROM '.$this->_tblCou.' AS c';
		
		//We build the query with %s placeholders for the values, then insert the values using wpdb::prepare
		//So we need an array to hold all our values to replace the placeholders with
		$aValsToInsertInQuery = array();
		
		if (!empty($oFetchInfoFor->currenciesBySymbol) || !empty($oFetchInfoFor->currencies)) {
			$sSQL .= ' INNER JOIN ( SELECT format_pref FROM '.$this->_tblCur.' WHERE';
			if (!empty($oFetchInfoFor->currenciesBySymbol)) {
				$sSQL .= ' (symbol IN ('.str_repeat('%s,',count($oFetchInfoFor->currenciesBySymbol)-1).'%s) AND symbol_pref = 1)';
				$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->currenciesBySymbol);
			}
			if (!empty($oFetchInfoFor->currencies)) {
				if (!empty($oFetchInfoFor->currenciesBySymbol)) {
					$sSQL .= ' OR';
				}
				$sSQL .= ' code IN ('.str_repeat('%s,',count($oFetchInfoFor->currencies)-1).'%s)';
				$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->currencies);
			}
			$sSQL .= ') AS x ON c.country = x.format_pref';
			if (!empty($oFetchInfoFor->countries)) {
				$sSQL .= ' UNION SELECT c.country, c.currency, c.dec_point, c.thousands_sep, c.symbol_pos
				FROM '.$this->_tblCou.' AS c
				WHERE c.country IN ('.str_repeat('%s,',count($oFetchInfoFor->countries)-1).'%s)';
				$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->countries);
			}
		//only countries
		} elseif (!empty($oFetchInfoFor->countries)) {
			$sSQL .= ' WHERE c.country IN ('.str_repeat('%s,',count($oFetchInfoFor->countries)-1).'%s)';
				$aValsToInsertInQuery = array_merge($aValsToInsertInQuery, $oFetchInfoFor->countries);
		//nothing
		} else {
			return;
		}
		$sSQL = $this->_wpdb->prepare($sSQL, $aValsToInsertInQuery);
		$aResults = $this->_wpdb->get_results($sSQL, OBJECT_K);
		foreach ($aResults as &$oResult) {
			$oResult->symbol_pos = (bool) $oResult->symbol_pos;
		}
		return $aResults;
	}
}
//Sever and PHP geoIP extension don't need WP, but we still need WP core loaded for db access
//Prevent all of WP loading
if (defined('SHORTINIT')) {
	return;
}
define( 'SHORTINIT', true );
//Load WordPress Core
require_once( '../../../../../wp-load.php' );
//formatting and meta are needed for fetching static rates post meta value
require_once( ABSPATH . WPINC . '/formatting.php' );
require_once( ABSPATH . WPINC . '/meta.php' );
require_once( dirname(__FILE__).'/../../lib/Xoogu/CountryFromIP/ICountryFromIP.php');
require_once( dirname(__FILE__).'/../../lib/Xoogu/CountryFromIP/CountryFromIP.php');
$xccja = new \Xoogu\CurrencyConversion\Ajax_Responder($GLOBALS['wpdb'], new \Xoogu\CountryFromIP\CountryFromIP());
$xccja->ajaxResponse();
