<?php
namespace Xoogu\CountryFromIP;
class Admin {
	//@var string holds the geo func in use (note this is different to the main CountryFromIP class as we don't split IP2Location methods here)
	protected $_sGeoFunc;
	//@var array holds any error messages to be printed
	protected $_aErrors;
	//@var array holds any success messages to be printed
	protected $_aSuccesses;
	
	public function __construct()
	{
        //localise
		load_plugin_textdomain('country-from-ip', false, plugin_basename(__FILE__).'/lang/');
		add_action('admin_init', array($this, 'admin_init'));
		add_action('admin_notices', array($this, 'adminMsg'));
		//load settings - use individual blog setting if available, otherwise look for network setting
		$sGeoFunc = get_option('countryFromIp-geoFunc');
		$this->_sGeoFunc = $sGeoFunc ? $sGeoFunc : get_site_option('countryFromIp-geoFunc');
	}
	
	public function admin_init()
	{
		//save settings
		if (isset($_POST['countryFromIp-geoFunc'])) {
			//check nonce & referrer
			if (wp_verify_nonce($_POST['_countryFromIP_nonce'], 'save_settings') !== 1){
				wp_die(__('A valid nonce was not found.','country-from-ip') );
			} else {
				$this->saveSettings($_POST['countryFromIp-geoFunc']);
			}
		}
		if(isset($_POST['installGeoIP'])) {
			//try to install the GeoIP plugin
			$this->_installAndUseGeoIP();
		}
	}
	
	
	/**
	 *Displays admin messages on settings pages
	 */
    public function adminMsg()
	{
		if (!empty($this->_aErrors)) {
			echo '<div class="error notice">';
			foreach ($this->_aErrors as $msg) {
				echo '<p>'.$msg.'</p>';
			}
			echo '</div>';
		}
		if (!empty($this->_aSuccesses)) {
			echo '<div class="updated notice">';
			foreach ($this->_aSuccesses as $msg) {
				echo '<p>'.$msg.'</p>';
			}
			echo '</div>';
		}
	}

	
	/**
     *Detects installed methods of getting the country code of a visitor
     *@param bool $network Whether to check if the method is available across a multisite network or just a single site
     *@return array List of supported methods
     */
    public function detectGeoIP($network=false){
        //cache result - switch_blog for WPMU does not modify plugins loaded so this func will always give the same result for each blog
        static $funcs;
        if(is_array($funcs)){
            return $funcs;
        }
        $funcs=array();
        if(isset($_SERVER['GEOIP_COUNTRY_CODE'])){
            $funcs['server'] = __('Server has GeoIP extension installed','country-from-ip');
        }
        //PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
        if(extension_loaded('geoip')){
            $funcs['php'] = __('PHP GeoIP extension','country-from-ip');
        }
		//GeoIP detection plugin Maxmind GeoLite-city (doesn't require the plugin active)
		if (file_exists(ABSPATH.'wp-content/uploads/GeoLite2-City.mmdb') && file_exists(ABSPATH.'wp-content/plugins/geoip-detect/vendor/maxmind-db/reader/src/MaxMind/Db/Reader.php')) {
			$funcs['geoip-detect-geolite-city'] = __('GeoIP Detection plugin MaxMind GeoLite City database', 'country-from-ip');
		}
		//IP2Location plugins
		$ip2LocationVersions = array('tags', 'variables', 'country-blocker', 'redirection');
		foreach ($ip2LocationVersions as $type) {
			//We don't need to check if the plugin is active, only that the IP2Location class file exists and database name is set
			//technically speaking, we should probably check get_site_option if $network is true. But ip2location plugins only use get / set option
			if ((file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$type.'/ip2location.class.php') || file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$type.'/class.IP2Location.php')) && get_option( 'ip2location_'.str_replace('-', '_', $type).'_database' )) {
				$funcs['ip2location-'.$type] = sprintf(__('IP2Location %s plugin','country-from-ip'),$type);
			}
		}
        //GeoIP Detection plugin - https://wordpress.org/plugins/geoip-detect/
        if ( ($network ? is_plugin_active_for_network('geoip-detect/geoip-detect.php') : is_plugin_active('geoip-detect/geoip-detect.php')) ) {
			if (function_exists('geoip_detect2_get_info_from_current_ip')) {
				$funcs['geoip-detect2'] = __('GeoIP Detection plugin v2','country-from-ip');
			}
        }
        //Try ip2nation table - http://wordpress.org/plugins/wp-ip2nation-installer/
        if($GLOBALS['wpdb']->get_var("SHOW TABLES LIKE 'ip2nation'")) {
            $funcs['ip2nation'] = __('ip2nation database table','country-from-ip');
        }
        //move the pointer to first item
        reset($funcs);
        return $funcs;
    }
	
	
	/**
	 *Detects which GeoIP methods are available and saves the first one as the selected GeoIP method to use
	 *@return Whether a GeoIP method was detected and saved as the method to use
	 */
	public function detectAndSetGeoIP()
	{
		//Get list of available geo funcs on this blog
		$aGeoFuncs = $this->detectGeoIP();
		if (!empty($aGeoFuncs)) {
			//if we already have a valid geo func set, then don't do anything
			if (!empty($aGeoFuncs[$this->_sGeoFunc])){
				return false;
			}
			//Otherwise use the first (preferred) geo func in the list
			return $this->saveSettings(key($aGeoFuncs));
		}
		return false;
	}
	
	
	/**
	 *Checks for supported GeoIP plugins that are installed but not activated
	 *@return array Array of GeoIP plugin names that are installed but not activated
	 *Format of array is "id used in plugin list in wordpress admin panel" => "plugin nice name"
	 */
	public function detectGeoIPinstalled()
	{
		//$installed_plugins = get_plugins();
		$installed = array();
		//go through them manually as id used in plugin list for geo-detect plugin is geo-detection (not geo-detect)
		if (file_exists(WP_PLUGIN_DIR."/geoip-detect/geoip-detect.php")) {
			$installed['geoip-detection'] = __('GeoIP Detection plugin','country-from-ip');
		}
		$ip2LocationVersions = array('tags', 'variables', 'country-blocker', 'redirection');
		foreach ($ip2LocationVersions as $type) {
			//technically speaking, we should probably check get_site_option if $network is true. But ip2location plugins only use get / set option
			if ((file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$type.'/ip2location.class.php') || file_exists(ABSPATH.'wp-content/plugins/ip2location-'.$type.'/class.IP2Location.php')) && !get_option( 'ip2location_'.str_replace('-', '_', $type).'_database' )) {
				$installed['ip2location-'.$type] = sprintf(__('IP2Location %s plugin','country-from-ip'),$type);
			}
		}
		return $installed;
	}
	
	
	/**
	 *Prints the form field for choosing the geoIP method, for use on plugin settings page
	 *@param bool $network Whether we're on the network settings page or an inividual blog settings page
	 */
	public function printFormFields($network)
	{
		$sSelectedGeoFunc = !empty($_POST['countryFromIp-geoFunc']) ? $_POST['countryFromIp-geoFunc'] : $this->_sGeoFunc;
		$aGeoFuncs = $this->detectGeoIP($network);
		if (empty($aGeoFuncs)) {
			if (($installed = $this->detectGeoIPinstalled()) && !empty($installed)) {
				foreach ($installed as $pluginId => $plugin_nicename) {
					echo '<p>'.sprintf(__('It looks like you have the %s installed but not activated. You will need to <a href="%s">activate a geo detection plugin</a> if you want to take advantage of this plugin\'s automatic conversion to the visitor\'s currency','country-from-ip'), $plugin_nicename, ($network ? network_admin_url('plugins.php') : admin_url('plugins.php')).'#'.$pluginId).'</p>';
				}
			}
			//display the button to install the GeoIP Detection plugin
			/*elseif(current_user_can('activate_plugins')){
				//button to auto install and activate the GeoIP Detection plugin - see $this->_installGeoIP()
				submit_button( __('Install GeoIP Detection plugin','country-from-ip'), 'secondary', 'installGeoIP' );
				//text link so user can download and install the GeoIP plugin manually if they wish
				echo '<p>'.sprintf(__('Or download and install manually from %s.','country-from-ip'),'<a href="https://wordpress.org/plugins/geoip-detect/">https://wordpress.org/plugins/geoip-detect/</a>').'</p>';
			}*/elseif(current_user_can('activate_plugins')){
				echo '<p>'.sprintf(__('It looks like you don\'t have any supported GeoIP lookup methods available. You can install a supported GeoIP plugin such as the <a href="%s">GeoIP Detection</a> or <a href="%s">IP2Location Tags</a> plugins.','country-from-ip'), (is_network_admin() ? network_admin_url('plugin-install.php?tab=plugin-information&plugin=geoip-detect') : admin_url('plugin-install.php?tab=plugin-information&plugin=geoip-detect')), (is_network_admin() ? network_admin_url('plugin-install.php?tab=plugin-information&plugin=ip2location-tags') : admin_url('plugin-install.php?tab=plugin-information&plugin=ip2location-tags'))).'</p>';
			}
		}
		?>
		<label for="geoFunc"><?php _e('Geo IP method:','country-from-ip');?></label>
		<select id="geoFunc" name="countryFromIp-geoFunc">
		<?php
		array_unshift($aGeoFuncs, '');
		foreach($aGeoFuncs as $key => $name){
			echo '<option value="'.$key.'"'.( ($sSelectedGeoFunc==$key) ? ' selected="selected"' : '').'>'.$name.'</option>';
		}
		?></select><?php
		wp_nonce_field('save_settings', '_countryFromIP_nonce');
	}
	
	
	/**
	 *Validates and saves the given geoFunc method
	 *@param string $sGeoFunc The geoFunc method to save as the default
	 *@return bool Whether the setting was saved or not
	 */
	public function saveSettings($sGeoFunc)
	{
		if ($this->validateSettings($sGeoFunc) === true) {
			if (is_network_admin()) {
				update_site_option('countryFromIp-geoFunc', $sGeoFunc);
				//delete any old options saved for individual blogs
				self::uninstallBlogs();
			} else {
				update_option('countryFromIp-geoFunc', $sGeoFunc);
			}
			return true;
		}
		return false;
	}
	
	
	/**
	 *Removes the settings from the db
	 */
	public static function uninstall()
	{
		delete_site_option('countryFromIp-geoFunc');
		self::uninstallBlogs();
	}
	
	
	/**
	 *Removes the per blog settings from the db
	 */
	public static function uninstallBlogs()
	{
		global $wpdb;
		$current = $wpdb->blogid;
		// Loop through the blogs
		$blogids = $wpdb->get_col('SELECT blog_id FROM '.$wpdb->blogs);
		foreach ($blogids as $blog_id) {
			switch_to_blog($blog_id);
			delete_option('countryFromIp-geoFunc');
		}
		//switch back to the original blog
		switch_to_blog($current);
	}
	
	
	/**
	 *Validates that the given geoFunc method is valid
	 *@param string $sGeoFunc The geoFunc method to check
	 *@return bool Whether the setting is valid or not
	 */
	public function validateSettings($sGeoFunc)
	{
		if(!current_user_can('manage_options')){
            wp_die(__('Sorry, but you must have the manage_options capability to view and edit settings. See <a href="http://codex.wordpress.org/Roles_and_Capabilities">Wordpress Codex: Roles and Capabilities</a> for more information.','country-from-ip') );
        }
        
        //geoFunc
        if (!empty($sGeoFunc)) {
            $aGeoFuncs = $this->detectGeoIP();
            if (empty($aGeoFuncs[$sGeoFunc])) {
                $this->_aErrors['geoFunc']= sprintf(__('%s is not a valid Geo IP method for this site','country-from-ip'), $sGeoFunc);
				return false;
            }
        }
		return true;
	}
	
	/**
	 *Allows checking what the geoIP method is / that a geoIP method is set
	 *@return string The set geoIP method
	 */
	public function getGeoMethod()
	{
		return $this->_sGeoFunc;
	}
	
    /**
     *Installs the Wordpress GeoIP Detection plugin
     */
    protected function _installAndUseGeoIP(){
		//try and install the plugin
        if (!($installed = $this->_installGeoIP())) {
			//Should be either false or a WP_Error object if something went wrong
			//the function will return false if the filesystem credentials form needs to be displayed
			//in which case, we don't need to display our settings form.
			return;
		}
		elseif( is_wp_error($installed) ){
			$this->_aErrors = $installed->get_error_messages();
			return;
		} 
		elseif($installed === true){
		 	$this->aSuccesses = __('GeoIP Detection plugin installed and activated successfully.','country-from-ip');
		
			//update the plugin settings to use the GeoIp Detection WP plugin
			$this->_sGeoFunc='geoip-detect2';
			if (is_network_admin()) {
				update_site_option('countryFromIp-geoFunc', 'geoip-detect2');
			} else {
				update_option('countryFromIp-geoFunc', 'geoip-detect2');
			}
			//update the POST so that the form will show with the GeoIp Detection WP plugin selected as the geoFunc method
			$_POST['countryFromIp-geoFunc']='geoip-detect2';
			return true;
		}
    }
	
	
	/**
     *Installs the Wordpress GeoIP Detection plugin
     */
    protected function _installGeoIP()
	{
        //check the user has the necessary privileges
        if(!current_user_can('activate_plugins')){
            return wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.','country-from-ip' ) );
        }
		if (is_network_admin() && !is_super_admin()) {
			return wp_die( __( 'You do not have sufficient permissions to network activate the plugin.','country-from-ip' ) );
		}
		
        //get the path the plugin will be installed to
        $pluginPath=WP_PLUGIN_DIR.'/geoip-detect';
        $pluginFile=$pluginPath.'/geoip-detect.php';
        
        //if the plugin is already activated, then something has gone wrong
        if(is_plugin_active(plugin_basename($pluginFile))){
            throw new Exception(__('Hmm... The install GeoIP detection plugin button should only appear if the plugin is not already activated. But it appears that the plugin is already activated! How did this happen?!','country-from-ip'));
            return;
        }
        
        //if the plugin doesn't exist, download it
        if(!is_dir($pluginPath)){
        
            //http://codex.wordpress.org/Filesystem_API
            //check we can write to the filesytem as the user - if we can't a form asking for FTP details will automatically be output
            $url = wp_nonce_url( (is_network_admin() ? network_admin_url('settings.php?page='.$_GET['page']) : get_admin_url(null, 'options-general.php?page='.$_GET['page'])), 'installGeoIP' );
            //form fields won't be passed correctly. This is a bug in WP - http://core.trac.wordpress.org/ticket/19643
            if (false === ($creds = request_filesystem_credentials($url, '', false, WP_PLUGIN_DIR, array('options' => array('from', 'dec_places', 'updateFreq', 'static', 'format'))) ) ) {
                return false; //If we can't write to the filesystem as the user, then don't continue
            }
            //Make sure the credentials actually work
            if ( ! WP_Filesystem($creds) ) {
                //If not, display the form asking for FTP details again
                request_filesystem_credentials($url, '', true, WP_PLUGIN_DIR, array('options' => array('from', 'dec_places', 'updateFreq', 'static', 'format')));
                return false;
            }
            
            //Use functions provided by wp-admin/includes/file.php
            //download the plugin
            $zip = download_url('https://downloads.wordpress.org/plugin/geoip-detect.zip');
			if (is_wp_error($zip)) {
				return $zip;
			}
            //unzip to the plugins directory
            $res = unzip_file($zip, WP_PLUGIN_DIR.'/');
            //delete the zip file
            unlink($zip);
			//wordpress will only activate the plugin if it is in its list of installed plugins.
			//Since this list will be cached from before we have just installed the plugin, we
			//need to empty the list from cache. Then WP will rebuild the installed plugin list
			//and see that the plugin is there for it to activate in the next step.
            wp_cache_delete('plugins', 'plugins');
        }
        if (is_network_admin()) {
			$res = activate_plugin($pluginFile, '', true);
		} else {
			$res = activate_plugin($pluginFile);
		}
		if( is_wp_error($res) ){
            return $res;
        }
        
        return true;
    }
}
